; 1dotVbatGaugeA.ASM: Pic10F22x + RGBLED(Common Anode) = 1 dot bargraph for battery gauge
; BENABADJI Salim - ORAN - August. 10th, 2019

;controller	equ	10F220
controller	equ	10F222

	if	controller == 10F220
		List P = 10F220
#include <P10F220.inc>
	endif
	
	if	controller == 10F222
		List P = 10F222
#include <P10F222.inc>
	endif

;WATCHDOG on
	__CONFIG _MCLRE_OFF & _CP_OFF & _WDT_ON & _MCPU_OFF & _IOFSCS_4MHZ

;----------------------------- define hardware -------------------------------
	
RLED	EQU		0	 ; assign red  LED to output GP0
GLED	EQU		1	 ; assign green LED to output GP1
BLED	EQU		2	 ; assign blue LED to output GP2 

;----------------------------- define constants ------------------------------

	CONSTANT  CBLOCK16 = 0x10 ; 16 bytes RAM for PIC10F200/204/220
	CONSTANT  CBLOCK23 = 0x09 ; 23 bytes RAM for PIC10F222
	;CONSTANT  CBLOCK24 = 0x08 ; 24 bytes RAM for PIC10F202/206

;REF06V	EQU	.59			; internal band gap 0.6V
;REFCST	EQU	.15045		; = 59 * 255
; then do REFCST / meas => Vdd * 100

;meas thresholds
TRIP1	 EQU	.50			; correspond to Vdd = 3,01 [V]
TRIP2	 EQU	.43			; correspond to Vdd = 3,50 [V]
TRIP3	 EQU	.36			; correspond to Vdd = 4,18 [V]

;------------------------------ define variables -----------------------------

	;CBLOCK	CBLOCK16	; if PIC10F220	
	CBLOCK	CBLOCK23	; if PIC10F222	

;delay
	cpt1, cpt2			; counter for delays
;measure
	meas				; 8-bit value from internal ADC module

	ENDC
;3 bytes used

;////////////////////////////////////////////////////////////////////////
;								MACROS
;////////////////////////////////////////////////////////////////////////

;------------------------------------------------------------------------
; if (file >= val) goto jmpOk ;
;------------------------------------------------------------------------
SIregGEval8b	macro	file, val, jmpOk
	MOVLW	val
	SUBWF	file, w		; w = file - val
	BTFSC	STATUS, C
	GOTO	jmpOk		; yes
	endm

;------------------------------------------------------------------------
; flash LED N, during X ms 
;------------------------------------------------------------------------
FLASH1LED	macro	ledN, valX
	MOVLW	valX
	MOVWF	cpt2
	BCF		GPIO, ledN
	CALL	Delay1ms
	BSF		GPIO, ledN
	DECFSZ	cpt2, F
	GOTO	$-4
	GOTO	doSleep
	endm

;------------------------------------------------------------------------
; flash LEDs N+M, during X ms 
;------------------------------------------------------------------------
FLASH2LED	macro	ledN, ledM, valX
	MOVLW	valX
	MOVWF	cpt2
	RRF		cpt2, F
	BCF		GPIO, ledN
	CALL	Delay1ms
	BSF		GPIO, ledN
	BCF		GPIO, ledM
	CALL	Delay1ms
	BSF		GPIO, ledM
	DECFSZ	cpt2, F
	GOTO	$-7
	GOTO	doSleep
	endm

;/////////////////////////////////////////////////////////////////////////////
	ORG		0			; address start of program memory
	GOTO	Setup
;/////////////////////////////////////////////////////////////////////////////

;-----------------------------------------------------------------------------
; precise delay of 1ms
;-----------------------------------------------------------------------------
Delay1ms				; 2 us
	MOVLW	.248		; 1 us
	MOVWF	cpt1		; 1 us
Delay
	NOP					; 1 us	\
	DECFSZ	cpt1, F		; 1 us	 |=> 4 * 248 = 992 us = 0.992 ms delay loop
	GOTO	Delay		; 2 us	/
	GOTO	$+1			; 2 us
	RETLW	0			; 2 us

;-----------------------------------------------------------------------------
; analog measure, 8-bit resolution
;-----------------------------------------------------------------------------
meas8bit
	MOVWF	ADCON0

	MOVLW	.8
	MOVWF	cpt1		; cpt1 = 32 us delay before acquisition (quartz 4 MHz)
	NOP					; 1 us
	DECFSZ	cpt1, F		; 1 us
	GOTO	$-2			; internal loop = 4 * 8 = 32 us

	bsf		ADCON0, GO	; initiate one A/D conversion
	btfsc	ADCON0, GO	; A/D conversion complete ?
	goto	$-1			; no

	movf	ADRES, W	; yes, then retreive measure
	movwf	meas		; store it in meas

	CLRF	ADCON0		; AN0, AN1 digitals, ADC off => low power consumption
	RETLW	0

;=============================================================================
Setup		        	; Beginning of the program

	BTFSS	STATUS, NOT_TO	; <4> cleared upon a WDT reset
	GOTO	fromWDTreset

;trim the internal oscillator accuracy to 1% at 4 MHz
	MOVLW	0x1A		; particular calibration value for my PIC10F220 sample
	MOVWF	OSCCAL		; trim the internal oscillator accuracy to 1% at 4 MHz

;turn off ADC (and set AN0 & AN1 as ditigal pins)
	CLRF	ADCON0		; AN0, AN1 digitals, ADC off => low power consumption

fromWDTreset ;----------------------------------------
; after WDT reset, register OPTION = b'1111 1111', so:
;cfg OPTION
	MOVLW 	b'11001110'	; no wake-up, no pull-up, <5>=0:use GP2, not TOCKI
	OPTION				; and WDT prescaler 1:64 => timeout # 1.15 s

;cfg GPIO
	MOVLW	b'11111000'	; (GP2,GP1,GP0) outputs
	TRIS	GPIO
	CLRF	GPIO		; all outputs LOW
	comf	GPIO, F		; all outputs HIGH, to light off RGB LED (Common Anode)
	
;pick meas
	MOVLW	b'00001101'	; select internal 0.6V ref, ADC on
	CALL	meas8bit

;meas >= TRIP1 ? => Vdd < 3.01 [V]
	SIregGEval8b	meas, TRIP1, batFlat
;meas >= TRIP2 ? => 3.01 < Vdd < 3.50 [V]
	SIregGEval8b	meas, TRIP2, batLow
;meas >= TRIP3 ? => 3.50 < Vdd < 4.18 [V]
	SIregGEval8b	meas, TRIP3, batFull

;overcharge
	FLASH1LED	BLED, .250	; flash blue LED (250ms) then goto sleep

batFlat
	FLASH1LED	RLED, .250	; flash red LED (250ms) then goto sleep

batLow
	FLASH2LED	RLED, GLED, .250 ; flash red/green = yellow LED (250ms) then goto sleep

batFull
	FLASH1LED	GLED, .250	; flash green LED (250ms) then goto sleep
	
doSleep
;deadtime of #1.15 s
;after CLRWDT, prescaler is cleared ; so reconfig. OPTION for WDT 1:64
	MOVLW  b'11001110'	; Disable wake-up, no pull-up, <5>=0:use GP2, not TOCKI
	OPTION				; and WDT prescaler 1:64 => timeout # 1.15 s
;veilleuse
	sleep				; sleep clears WDT

;-----------------------------------------------------------------------------
	END
